@@ -17,7 +17,6 @@
 
 from __future__ import division, print_function, absolute_import
 
-import hashlib
 from collections import defaultdict
 
 import hypothesis.internal.conjecture.utils as cu
@@ -25,30 +24,17 @@ from hypothesis.errors import NoExamples, NoSuchExample, Unsatisfiable, \
     UnsatisfiedAssumption
 from hypothesis.control import assume, reject, _current_build_context
 from hypothesis._settings import note_deprecation
-from hypothesis.internal.compat import hrange, qualname, bit_length, \
-    str_to_bytes, int_from_bytes
+from hypothesis.internal.compat import hrange, bit_length
 from hypothesis.utils.conventions import UniqueIdentifier
 from hypothesis.internal.lazyformat import lazyformat
 from hypothesis.internal.reflection import get_pretty_function_description
+from hypothesis.internal.conjecture.utils import LABEL_MASK, \
+    combine_labels, calc_label_from_cls, calc_label_from_name
 
 calculating = UniqueIdentifier('calculating')
 
-
-LABEL_MASK = 2 ** 64 - 1
-
-
-def calc_label(cls):
-    name = str_to_bytes(qualname(cls))
-    hashed = hashlib.md5(name).digest()
-    return int_from_bytes(hashed[:8])
-
-
-def combine_labels(*labels):
-    label = 0
-    for l in labels:
-        label = (label << 1) & LABEL_MASK
-        label ^= l
-    return label
+MAPPED_SEARCH_STRATEGY_DO_DRAW_LABEL = calc_label_from_name(
+    'another attempted draw in MappedSearchStrategy')
 
 
 def one_of_strategies(xs):
@@ -67,7 +53,6 @@ class SearchStrategy(object):
     the public API and their behaviour may change significantly between
     minor version releases. They will generally be stable between patch
     releases.
-
     """
 
     supports_find = True
@@ -99,7 +84,6 @@ class SearchStrategy(object):
         Michael D., Celeste Hollenbeck, and Matthew Might. "On the complexity
         and performance of parsing with derivatives." ACM SIGPLAN Notices 51.6
         (2016): 224-236.
-
         """
         cache_key = 'cached_' + name
         calculation = 'calc_' + name
@@ -161,6 +145,21 @@ class SearchStrategy(object):
             else:
                 needs_update = None
 
+            def recur2(strat):
+                def recur_inner(other):
+                    try:
+                        return forced_value(other)
+                    except AttributeError:
+                        pass
+                    listeners[other].add(strat)
+                    try:
+                        return mapping[other]
+                    except KeyError:
+                        needs_update.add(other)
+                        mapping[other] = default
+                        return default
+                return recur_inner
+
             count = 0
             seen = set()
             while needs_update:
@@ -183,20 +182,7 @@ class SearchStrategy(object):
                 to_update = needs_update
                 needs_update = set()
                 for strat in to_update:
-                    def recur(other):
-                        try:
-                            return forced_value(other)
-                        except AttributeError:
-                            pass
-                        listeners[other].add(strat)
-                        try:
-                            return mapping[other]
-                        except KeyError:
-                            needs_update.add(other)
-                            mapping[other] = default
-                            return default
-
-                    new_value = getattr(strat, calculation)(recur)
+                    new_value = getattr(strat, calculation)(recur2(strat))
                     if new_value != mapping[strat]:
                         needs_update.update(listeners[strat])
                         mapping[strat] = new_value
@@ -250,7 +236,6 @@ class SearchStrategy(object):
         exploration of the API, not for any sort of real testing.
 
         This method is part of the public API.
-
         """
         context = _current_build_context.value
         if context is not None:
@@ -300,7 +285,6 @@ class SearchStrategy(object):
                 random=random,
                 settings=settings(
                     max_shrinks=0,
-                    max_iterations=1000,
                     database=None,
                     verbosity=Verbosity.quiet,
                 )
@@ -319,7 +303,6 @@ class SearchStrategy(object):
         from this strategy and then calling pack() on the result, giving that.
 
         This method is part of the public API.
-
         """
         return MappedSearchStrategy(
             pack=pack, strategy=self
@@ -331,7 +314,6 @@ class SearchStrategy(object):
         strategy(expand(x))
 
         This method is part of the public API.
-
         """
         from hypothesis.searchstrategy.flatmapped import FlatMapStrategy
         return FlatMapStrategy(
@@ -345,7 +327,6 @@ class SearchStrategy(object):
         Unsatisfiable.
 
         This method is part of the public API.
-
         """
         return FilteredStrategy(
             condition=condition,
@@ -361,7 +342,6 @@ class SearchStrategy(object):
         of this strategy or the other strategy.
 
         This method is part of the public API.
-
         """
         if not isinstance(other, SearchStrategy):
             raise ValueError('Cannot | a SearchStrategy with %r' % (other,))
@@ -371,7 +351,6 @@ class SearchStrategy(object):
         """Throw an exception if the strategy is not valid.
 
         This can happen due to lazy construction
-
         """
         if self.validate_called:
             return
@@ -384,7 +363,7 @@ class SearchStrategy(object):
             self.validate_called = False
             raise
 
-    LABELS = {}
+    LABELS = {}  # type: dict
 
     @property
     def class_label(self):
@@ -393,7 +372,7 @@ class SearchStrategy(object):
             return cls.LABELS[cls]
         except KeyError:
             pass
-        result = calc_label(cls)
+        result = calc_label_from_cls(cls)
         cls.LABELS[cls] = result
         return result
 
@@ -426,7 +405,6 @@ class OneOfStrategy(SearchStrategy):
     The conditional distribution draws uniformly at random from some
     non-empty subset of these strategies and then draws from the
     conditional distribution of that strategy.
-
     """
 
     def __init__(self, strategies, bias=None):
@@ -505,7 +483,7 @@ class OneOfStrategy(SearchStrategy):
             self.element_strategies[i], label=self.branch_labels[i])
 
     def __repr__(self):
-        return ' | '.join(map(repr, self.original_strategies))
+        return 'one_of(%s)' % ', '.join(map(repr, self.original_strategies))
 
     def do_validate(self):
         for e in self.element_strategies:
@@ -528,7 +506,6 @@ class MappedSearchStrategy(SearchStrategy):
     strategy.
 
     Its parameter and distribution come from that other strategy.
-
     """
 
     def __init__(self, strategy, pack=None):
@@ -554,7 +531,7 @@ class MappedSearchStrategy(SearchStrategy):
     def do_validate(self):
         self.mapped_strategy.validate()
 
-    def pack(self, x):
+    def pack(self, x):  # type: ignore
         """Take a value produced by the underlying mapped_strategy and turn it
         into a value suitable for outputting from this strategy."""
         raise NotImplementedError(
@@ -564,7 +541,7 @@ class MappedSearchStrategy(SearchStrategy):
         for _ in range(3):
             i = data.index
             try:
-                data.start_example()
+                data.start_example(MAPPED_SEARCH_STRATEGY_DO_DRAW_LABEL)
                 result = self.pack(data.draw(self.mapped_strategy))
                 data.stop_example()
                 return result
@@ -627,8 +604,7 @@ class FilteredStrategy(SearchStrategy):
 
     @property
     def branches(self):
-        branches = [
+        return [
             FilteredStrategy(strategy=strategy, condition=self.condition)
             for strategy in self.filtered_strategy.branches
         ]
-        return branches
