@@ -36,12 +36,19 @@ from hypothesis.internal.compat import gcd, ceil, floor, hrange, \
 from hypothesis.internal.floats import next_up, next_down, is_negative, \
     float_to_int, int_to_float, count_between_floats
 from hypothesis.internal.charmap import as_general_categories
+from hypothesis.internal.cathetus import cathetus
 from hypothesis.internal.renaming import renamed_arguments
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.reflection import proxies, required_args
 from hypothesis.internal.validation import check_type, try_convert, \
     check_strategy, check_valid_size, check_valid_bound, \
-    check_valid_sizes, check_valid_integer, check_valid_interval
+    check_valid_sizes, check_valid_integer, check_valid_interval, \
+    check_valid_magnitude
+
+if False:
+    from typing import Any, Dict, Union, Sequence, Callable, Pattern, Tuple  # noqa
+    # See https://github.com/python/mypy/issues/3186 - numbers.Real is wrong!
+    Real = Union[int, float, Fraction, Decimal]
 
 __all__ = [
     'nothing',
@@ -175,6 +182,7 @@ NOTHING = Nothing()
 
 @cacheable
 def nothing():
+    # type: () -> SearchStrategy
     """This strategy never successfully draws a value and will always reject on
     an attempt to draw.
 
@@ -201,6 +209,7 @@ def just(value):
 
 @defines_strategy_with_reusable_values
 def none():
+    # type: () -> SearchStrategy
     """Return a strategy which only generates None.
 
     Examples from this strategy do not shrink (because there is only
@@ -209,7 +218,10 @@ def none():
     return just(None)
 
 
-def one_of(*args):
+def one_of(
+    *args  # type: Union[SearchStrategy, Sequence[SearchStrategy]]
+):
+    # type: (...) -> SearchStrategy
     """Return a strategy which generates values from any of the argument
     strategies.
 
@@ -240,6 +252,7 @@ def one_of(*args):
 @cacheable
 @defines_strategy_with_reusable_values
 def integers(min_value=None, max_value=None):
+    # type: (Real, Real) -> SearchStrategy
     """Returns a strategy which generates integers (in Python 2 these may be
     ints or longs).
 
@@ -293,6 +306,7 @@ def integers(min_value=None, max_value=None):
 @cacheable
 @defines_strategy
 def booleans():
+    # type: () -> SearchStrategy
     """Returns a strategy which generates instances of bool.
 
     Examples from this strategy will shrink towards False (i.e.
@@ -430,23 +444,10 @@ def floats(
         return result
 
 
-@cacheable
-@defines_strategy_with_reusable_values
-def complex_numbers():
-    """Returns a strategy that generates complex numbers.
-
-    Examples from this strategy shrink by shrinking their component real
-    and imaginary parts.
-    """
-    from hypothesis.searchstrategy.numbers import ComplexStrategy
-    return ComplexStrategy(
-        tuples(floats(), floats())
-    )
-
-
 @cacheable
 @defines_strategy
 def tuples(*args):
+    # type: (*SearchStrategy) -> SearchStrategy
     """Return a strategy which generates a tuple of the same length as args by
     generating the value at index i from args[i].
 
@@ -464,6 +465,7 @@ def tuples(*args):
 
 @defines_strategy
 def sampled_from(elements):
+    # type: (Union[Sequence, enum.Enum]) -> SearchStrategy
     """Returns a strategy which generates any value present in ``elements``.
 
     Note that as with :func:`~hypotheses.strategies.just`, values will not be
@@ -486,7 +488,7 @@ def sampled_from(elements):
     if len(values) == 1:
         return just(values[0])
     if hasattr(enum, 'Flag') and isclass(elements) and \
-            issubclass(elements, enum.Flag):
+            issubclass(elements, enum.Flag):  # type: ignore
         # Combinations of enum.Flag members are also members.  We generate
         # these dynamically, because static allocation takes O(2^n) memory.
         return sets(sampled_from(values), min_size=1).map(
@@ -497,9 +499,14 @@ def sampled_from(elements):
 @cacheable
 @defines_strategy
 def lists(
-    elements=None, min_size=None, average_size=None, max_size=None,
-    unique_by=None, unique=False,
+    elements=None,  # type: SearchStrategy
+    min_size=None,  # type: int
+    average_size=None,  # type: int
+    max_size=None,  # type: int
+    unique_by=None,  # type: Callable[..., Any]
+    unique=False,  # type: bool
 ):
+    # type: (...) -> SearchStrategy
     """Returns a list containing values drawn from elements with length in the
     interval [min_size, max_size] (no bounds in that direction if these are
     None). If max_size is 0 then elements may be None and only the empty list
@@ -537,7 +544,7 @@ def lists(
         return builds(list)
     if max_size == 0:
         return builds(list)
-    check_strategy(elements)
+    check_strategy(elements, 'elements')
     if unique:
         if unique_by is not None:
             raise InvalidArgument((
@@ -548,12 +555,6 @@ def lists(
             def unique_by(x):
                 return x
 
-    if min_size is None:
-        min_size = 0
-
-    if max_size is None:
-        max_size = float('inf')
-
     from hypothesis.searchstrategy.collections import ListStrategy, \
         UniqueListStrategy
     if unique_by is not None:
@@ -569,6 +570,7 @@ def lists(
 @cacheable
 @defines_strategy
 def sets(elements=None, min_size=None, average_size=None, max_size=None):
+    # type: (SearchStrategy, int, int, int) -> SearchStrategy
     """This has the same behaviour as lists, but returns sets instead.
 
     Note that Hypothesis cannot tell if values are drawn from elements
@@ -593,6 +595,7 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
 @cacheable
 @defines_strategy
 def frozensets(elements=None, min_size=None, average_size=None, max_size=None):
+    # type: (SearchStrategy, int, int, int) -> SearchStrategy
     """This is identical to the sets function but instead returns
     frozensets."""
     if elements is None:
@@ -648,6 +651,7 @@ def iterables(elements=None, min_size=None, average_size=None, max_size=None,
 
 @defines_strategy
 def fixed_dictionaries(mapping):
+    # type: (Dict[Any, SearchStrategy]) -> SearchStrategy
     """Generates a dictionary of the same type as mapping with a fixed set of
     keys mapping to strategies. mapping must be a dict subclass.
 
@@ -719,9 +723,15 @@ def streaming(elements):
 
 @cacheable
 @defines_strategy_with_reusable_values
-def characters(whitelist_categories=None, blacklist_categories=None,
-               blacklist_characters=None, min_codepoint=None,
-               max_codepoint=None, whitelist_characters=None):
+def characters(
+    whitelist_categories=None,  # type: Sequence[str]
+    blacklist_categories=None,  # type: Sequence[str]
+    blacklist_characters=None,  # type: Sequence[str]
+    min_codepoint=None,  # type: int
+    max_codepoint=None,  # type: int
+    whitelist_characters=None,  # type: Sequence[str]
+):
+    # type: (...) -> SearchStrategy
     """Generates unicode text type (unicode on python 2, str on python 3)
     characters following specified filtering rules.
 
@@ -812,6 +822,7 @@ def text(
     alphabet=None,
     min_size=None, average_size=None, max_size=None
 ):
+    # type: (SearchStrategy, int, int, int) -> SearchStrategy
     """Generates values of a unicode text type (unicode on python 2, str on
     python 3) with values drawn from alphabet, which should be an iterable of
     length one strings or a strategy generating such. If it is None it will
@@ -851,6 +862,7 @@ def text(
 @cacheable
 @defines_strategy
 def from_regex(regex):
+    # type: (Union[str, bytes, Pattern]) -> SearchStrategy
     """Generates strings that contain a match for the given regex (i.e. ones
     for which :func:`re.search` will return a non-None result).
 
@@ -885,6 +897,7 @@ def from_regex(regex):
 def binary(
     min_size=None, average_size=None, max_size=None
 ):
+    # type: (int, int, int) -> SearchStrategy
     """Generates the appropriate binary type (str in python 2, bytes in python
     3).
 
@@ -1170,9 +1183,11 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
                 'max_denominator=%r must be >= 1' % max_denominator)
 
         def fraction_bounds(value):
+            # type: (Fraction) -> Tuple[Fraction, Fraction]
             """Find the best lower and upper approximation for value."""
             # Adapted from CPython's Fraction.limit_denominator here:
             # https://github.com/python/cpython/blob/3.6/Lib/fractions.py#L219
+            assert max_denominator is not None
             if value is None or value.denominator <= max_denominator:
                 return value, value
             p0, q0, p1, q1 = 0, 1, 1, 0
@@ -1194,7 +1209,8 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
         _, min_value = fraction_bounds(min_value)
         max_value, _ = fraction_bounds(max_value)
 
-        if None not in (min_value, max_value) and min_value > max_value:
+        if min_value is not None and max_value is not None and \
+                min_value > max_value:
             raise InvalidArgument(
                 'There are no fractions with a denominator <= %r between '
                 'min_value=%r and max_value=%r' % bounds)
@@ -1337,7 +1353,12 @@ def decimals(min_value=None, max_value=None,
     return strat | sampled_from(special)
 
 
-def recursive(base, extend, max_leaves=100):
+def recursive(
+    base,  # type: SearchStrategy
+    extend,  # type: Callable[[SearchStrategy], SearchStrategy]
+    max_leaves=100,  # type: int
+):
+    # type: (...) -> SearchStrategy
     """base: A strategy to start from.
 
     extend: A function which takes a strategy and returns a new strategy.
@@ -1366,6 +1387,7 @@ def recursive(base, extend, max_leaves=100):
 
 @defines_strategy
 def permutations(values):
+    # type: (Sequence) -> SearchStrategy
     """Return a strategy which returns permutations of the collection
     ``values``.
 
@@ -1398,16 +1420,20 @@ def permutations(values):
     max_datetime='max_value',
 )
 def datetimes(
-    min_value=dt.datetime.min, max_value=dt.datetime.max,
-    timezones=none(),
-    min_datetime=None, max_datetime=None,
+    min_value=dt.datetime.min,  # type: dt.datetime
+    max_value=dt.datetime.max,  # type: dt.datetime
+    timezones=none(),  # type: SearchStrategy
+    min_datetime=None,  # type: dt.datetime
+    max_datetime=None,  # type: dt.datetime
 ):
+    # type: (...) -> SearchStrategy
     """A strategy for generating datetimes, which may be timezone-aware.
 
     This strategy works by drawing a naive datetime between ``min_datetime``
     and ``max_datetime``, which must both be naive (have no timezone).
 
-    ``timezones`` must be a strategy that generates tzinfo objects (or None,
+    ``timezones`` must be a strategy that generates
+    :class:`~python:datetime.tzinfo` objects (or None,
     which is valid for naive datetimes).  A value drawn from this strategy
     will be added to a naive datetime, and the resulting tz-aware datetime
     returned.
@@ -1418,9 +1444,9 @@ def datetimes(
         adjustments, etc.  This is intentional, as malformed timestamps are a
         common source of bugs.
 
-    :py:func:`hypothesis.extra.timezones` requires the ``pytz`` package, but
-    provides all timezones in the Olsen database.  If you also want to allow
-    naive datetimes, combine strategies like ``none() | timezones()``.
+    :py:func:`hypothesis.extra.pytz.timezones` requires the :pypi:`pytz`
+    package, but provides all timezones in the Olsen database.  If you want to
+    allow naive datetimes, combine strategies like ``none() | timezones()``.
 
     Alternatively, you can create a list of the timezones you wish to allow
     (e.g. from the standard library, ``datetutil``, or ``pytz``) and use
@@ -1466,6 +1492,7 @@ def dates(
     min_value=dt.date.min, max_value=dt.date.max,
     min_date=None, max_date=None,
 ):
+    # type: (dt.date, dt.date, dt.date, dt.date) -> SearchStrategy
     """A strategy for dates between ``min_date`` and ``max_date``.
 
     Examples from this strategy shrink towards January 1st 2000.
@@ -1486,9 +1513,13 @@ def dates(
     max_time='max_value',
 )
 def times(
-    min_value=dt.time.min, max_value=dt.time.max, timezones=none(),
-    min_time=None, max_time=None,
+    min_value=dt.time.min,  # type: dt.time
+    max_value=dt.time.max,  # type: dt.time
+    timezones=none(),  # type: SearchStrategy
+    min_time=None,  # type: dt.time
+    max_time=None,  # type: dt.time
 ):
+    # type: (...) -> SearchStrategy
     """A strategy for times between ``min_time`` and ``max_time``.
 
     The ``timezones`` argument is handled as for :py:func:`datetimes`.
@@ -1515,9 +1546,12 @@ def times(
     max_delta='max_value',
 )
 def timedeltas(
-    min_value=dt.timedelta.min, max_value=dt.timedelta.max,
-    min_delta=None, max_delta=None
+    min_value=dt.timedelta.min,  # type: dt.timedelta
+    max_value=dt.timedelta.max,  # type: dt.timedelta
+    min_delta=None,  # type: dt.timedelta
+    max_delta=None,  # type: dt.timedelta
 ):
+    # type: (...) -> SearchStrategy
     """A strategy for timedeltas between ``min_value`` and ``max_value``.
 
     Examples from this strategy shrink towards zero.
@@ -1589,7 +1623,92 @@ def composite(f):
     return accept
 
 
+@defines_strategy_with_reusable_values
+@cacheable
+def complex_numbers(min_magnitude=0, max_magnitude=None,
+                    allow_infinity=None, allow_nan=None):
+    """Returns a strategy that generates complex numbers.
+
+    This strategy draws complex numbers with constrained magnitudes.
+    The ``min_magnitude`` and ``max_magnitude`` parameters should be
+    non-negative :class:`~python:numbers.Real` numbers; values
+    of ``None`` correspond to zero and infinite values respectively.
+
+    If ``min_magnitude`` is positive or ``max_magnitude`` is finite, it
+    is an error to enable ``allow_nan``.  If ``max_magnitude`` is finite,
+    it is an error to enable ``allow_infinity``.
+
+    The magnitude contraints are respected up to a relative error
+    of (around) floating-point epsilon, due to implementation via
+    the system ``sqrt`` function.
+
+    Examples from this strategy shrink by shrinking their real and
+    imaginary parts, as :func:`~hypothesis.strategies.floats`.
+
+    If you need to generate complex numbers with particular real and
+    imaginary parts or relationships between parts, consider using
+    `builds(complex, ...) <hypothesis.strategies.builds>` or
+    `@composite <hypothesis.strategies.composite>` respectively.
+    """
+    check_valid_magnitude(min_magnitude, 'min_magnitude')
+    check_valid_magnitude(max_magnitude, 'max_magnitude')
+    check_valid_interval(min_magnitude, max_magnitude,
+                         'min_magnitude', 'max_magnitude')
+    if max_magnitude == float('inf'):
+        max_magnitude = None
+    if min_magnitude == 0:
+        min_magnitude = None
+
+    if allow_infinity is None:
+        allow_infinity = bool(max_magnitude is None)
+    elif allow_infinity and max_magnitude is not None:
+        raise InvalidArgument(
+            'Cannot have allow_infinity=%r with max_magnitude=%r' %
+            (allow_infinity, max_magnitude)
+        )
+    if allow_nan is None:
+        allow_nan = bool(min_magnitude is None and max_magnitude is None)
+    elif allow_nan and not (min_magnitude is None and max_magnitude is None):
+        raise InvalidArgument(
+            'Cannot have allow_nan=%r, min_magnitude=%r max_magnitude=%r' %
+            (allow_nan, min_magnitude, max_magnitude)
+        )
+    allow_kw = dict(allow_nan=allow_nan, allow_infinity=allow_infinity)
+
+    if min_magnitude is None and max_magnitude is None:
+        # In this simple but common case, there are no constraints on the
+        # magnitude and therefore no relationship between the real and
+        # imaginary parts.
+        return builds(complex, floats(**allow_kw), floats(**allow_kw))
+
+    @composite
+    def constrained_complex(draw):
+        # Draw the imaginary part, and determine the maximum real part given
+        # this and the max_magnitude
+        if max_magnitude is None:
+            zi = draw(floats(**allow_kw))
+            rmax = float('inf')
+        else:
+            zi = draw(floats(-max_magnitude, max_magnitude, **allow_kw))
+            rmax = cathetus(max_magnitude, zi)
+        # Draw the real part from the allowed range given the imaginary part
+        if min_magnitude is None or math.fabs(zi) >= min_magnitude:
+            zr = draw(floats(-rmax, rmax, **allow_kw))
+        else:
+            zr = draw(floats(cathetus(min_magnitude, zi), rmax, **allow_kw))
+        # Order of conditions carefully tuned so that for a given pair of
+        # magnitude arguments, we always either draw or do not draw the bool
+        # (crucial for good shrinking behaviour) but only invert when needed.
+        if min_magnitude is not None and draw(booleans()) and \
+                math.fabs(zi) <= min_magnitude:
+            zr = -zr
+        return complex(zr, zi)
+
+    return constrained_complex()
+
+
 def shared(base, key=None):
+    # type: (SearchStrategy, Any) -> SearchStrategy
     """Returns a strategy that draws a single shared value per run, drawn from
     base. Any two shared instances with the same key will share the same value,
     otherwise the identity of this strategy will be used. That is:
@@ -1667,6 +1786,7 @@ def choices():
 @cacheable
 @defines_strategy_with_reusable_values
 def uuids(version=None):
+    # type: (int) -> SearchStrategy
     """Returns a strategy that generates :class:`UUIDs <uuid.UUID>`.
 
     If the optional version argument is given, value is passed through
@@ -1786,6 +1906,7 @@ def data():
 
 
 def register_type_strategy(custom_type, strategy):
+    # type: (type, Union[type, Callable[[type], SearchStrategy]]) -> None
     """Add an entry to the global type-to-strategy lookup.
 
     This lookup is used in :func:`~hypothesis.strategies.builds` and
@@ -1815,6 +1936,7 @@ def register_type_strategy(custom_type, strategy):
 
 @cacheable
 def deferred(definition):
+    # type: (Callable[[], SearchStrategy]) -> SearchStrategy
     """A deferred strategy allows you to write a strategy that references other
     strategies that have not yet been defined. This allows for the easy
     definition of recursive and mutually recursive strategies.
